<?php


/*
This file contains the following things:

1. Classes to create heirarchial command and convert them to PRISM commands.
2. Functions to create variable names.
3. Global constants, such as number of pieces, number of agents, and number of blocks.

*/


// MISC UTILITY FUNCTIONS
function newl() { echo "\r\n";};

// CONSTANTS

$NO_AGENT = -1;
$SEEDER = 0;
$NUM_NORMAL_AGENTS = 2;
$ATTACKER = $NUM_NORMAL_AGENTS+1;
$TOTAL_NUM_AGENTS = $ATTACKER+1;

$MAX_WORK = 100000;


$NUM_PIECES = 2;
$NUM_BLOCKS = 2;

$MAX_PIECE = $NUM_PIECES-1;
$MAX_BLOCK = $NUM_BLOCKS-1;

// default request probability (prob that an agent requests a specific block)
$DEF_REQ_PROB = 0.9;
$SEND_BLOCK_PROB = 0.9;

//adds parentheses around every term
function convertResultString($str) {
	$arr = array_map("trim",split("&",$str));
	$ret = "(".join(") & (", $arr).")";
	return $ret;
}

function joinProbWithResult($arr) {
	return $arr[0].": ".$arr[1];
}

function fixDistributionStrings($arr) {
	return array($arr[0],convertResultString($arr[1]));
}

class Result {
	private $distribution = array();
	
	
	//2 kinds of parameters
	//1. fixed string aka "c'=2 & d'=1"
	//2. array of arrays aka [[.5, "c'=2 & d'=1], [.5, "a'=3]]
	function __construct($rep) {
	 if(is_string($rep)) {
		array_push($this->distribution,array(1.0,$rep));
	 } else { //it's an array
		foreach($rep as $elem) {
			array_push($this->distribution,array($elem[0],$elem[1]));
		}
	 }
	}
	
	//returns new Result with combined distribution
	public function combine($otherResult) {
		$newRep = array();
		foreach($this->distribution as $elem1) {
			foreach($otherResult->distribution as $elem2) {
				$newElem = NULL;
				if($elem1[1]==NULL) $newElem = $elem2[1];
				else if($elem2[1]==NULL) $newElem = $elem1[1];
				else $newElem = $elem1[1]." & ".$elem2[1];
				array_push($newRep,array("((".$elem1[0].") * (".$elem2[0]."))",$newElem));
			}
		}
		return new Result($newRep);
	}
	
	// returns distribution in result form
	public function toString() {
		
		$temp = array_map("joinProbWithResult", array_map("fixDistributionStrings",$this->distribution));
		return join(" + ", $temp);
	}
	
}

class TieredTransition {
	private $children = array();
	private $condition;
	private $result;
	private $par = NULL;

	//NULL for $cond means "else"
	//NULL for $result means "no results"
	function __construct($cond, $result) {
		$this->condition = $cond;
		if($result!=NULL)
			$this->result = new Result($result);
		else $this->result = NULL;
	}
	
	public function condition() { 
		$ret = $this->condition;
		if($ret==NULL) {
			//construct else condition
			$ret = "";
			$siblings = $this->par->children;
			$other_children = array();
			foreach($siblings as $child) {
				if($child!=$this) array_push($other_children,$child);
			}
			for($i=0; $i<count($other_children); $i++) {
				$child = $other_children[$i];
				$ret = $ret."!(".$child->condition. ")";
				if($i<count($other_children)-1) $ret = $ret." & ";
			}
			if($ret=="") $ret = "true";
		}
		return $ret; 
	}
	public function result() { return $this->result; }
	public function children() { return $this->children; }
	
	public function addChild($tt) {
		array_push($this->children,$tt);
		$tt->par = $this;
	}
	
	private function printTraverseTree($tt, $curr_condition, $curr_result) {
		if($curr_condition==NULL) $curr_condition = $tt->condition();
		else $curr_condition = $curr_condition . " & " . $tt->condition();
		if($tt->result()!=NULL) {
			if($curr_result==NULL) $curr_result = $tt->result();
			else $curr_result = $tt->result()->combine($curr_result);
		}
		if(count($tt->children())==0) {
			echo "[] ".$curr_condition." -> ".$curr_result->toString().";"; newl();
		}
		foreach($tt->children() as $child) {
			$this->printTraverseTree($child,$curr_condition, $curr_result);
		}
	}
	
	public function printTransitions() {
		$this->printTraverseTree($this,NULL,NULL);
	}
}

/*--------------- VARIABLE NAMES --------------------*/

function turnVar() {
	return "turn";
}

function counterVar() {
	return "counter";
}

function workVar($agentID) {
	return "ag{$agentID}_work_done";
}

function reqProbability($agentID) {
	global $DEF_REQ_PROB;
	return "{$DEF_REQ_PROB}";
}

function sendProbability($agentID) {
	global $SEND_BLOCK_PROB;
	return "{$SEND_BLOCK_PROB}";
}

function pieceStatusVar($agentID, $p) {
	return "ag{$agentID}_p{$p}_piece_status";
}

function blockStatusVar($agentID, $p, $l) {
	return "ag{$agentID}_p{$p}_b{$l}_block_status";
}

function blockStatusPortVar($agentID, $p, $l) {
	return "ag{$agentID}_p{$p}_b{$l}_block_status_port";
}

function reqFromVar($agentID, $p, $l) {
	return "ag{$agentID}_p{$p}_b{$l}_req_from";
}

function isChokedVar($agentMe, $agentOther) {
	return "ag{$agentMe}_ag{$agentOther}_is_choked";
}

function isInterestedInVar($agentMe, $agentOther) {
	return "ag{$agentMe}_ag{$agentOther}_is_interested_in";
}

function isChokingVar($agentMe, $agentOther) {
	return "ag{$agentMe}_ag{$agentOther}_is_choking";
}

function thinksInterestedVar($agentMe, $agentOther) {
	return "ag{$agentMe}_ag{$agentOther}_thinks_interested";
}

function thinksInterestedPortVar($agentMe, $agentOther) {
	return "ag{$agentMe}_ag{$agentOther}_thinks_interested_port";
}

function thinksHasPieceVar($agentMe, $agentOther, $p) {
	return "ag{$agentMe}_ag{$agentOther}_p{$p}_thinks_has_piece";
}

function thinksHasPiecePortVar($agentMe, $agentOther, $p) {
	return "ag{$agentMe}_ag{$agentOther}_p{$p}_thinks_has_piece_port";
}

function isReqVar($agentMe, $agentOther, $p, $l) {
	return "ag{$agentMe}_ag{$agentOther}_p{$p}_b{$l}_is_req";
}

function isReqPortVar($agentMe, $agentOther, $p, $l) {
	return "ag{$agentMe}_ag{$agentOther}_p{$p}_b{$l}_is_req_port";
}

function reqActionStateVar($agentMe, $agentOther, $p, $l) {
	return "ag{$agentMe}_ag{$agentOther}_p{$p}_b{$l}_req_action_state";
}

function sendActionStateVar($agentMe, $agentOther, $p, $l) {
	return "ag{$agentMe}_ag{$agentOther}_p{$p}_b{$l}_send_action_state";
}

function justDownloadedPieceVar($agent) {
	return "ag{$agent}_just_downloaded_piece";
}

function justDownloadedBlockVar($agent, $piece) {
	return "ag{$agent}_p{$piece}_just_downloaded_block";
}


?>